

//***********************************************************************
//*	Codigo para um Sistema Multi-Display e Multi-I/O (também
//	chamado de Sistema "Multi-PIO"), implementado para o Lab de
//	Garagem do Link a seguir:
//*
//*	link: http://labdegaragem.com/forum/topics/ligando-um-monte-de-componente-no-arduino
//*
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Definicoes para o Hardware usado neste Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Definicao do total de Displays, Saidas e Entradas Digitais
//	que sao implementados neste Sistema.
//
//	Obs.: para as Entradas e Saidas Digitais, o total especificado
//	para cada, deve ser obrigatoriamente um multiplo de 8, mesmo
//	que nem todas as Entradas e Saidas sejam usadas no Hardware.
//	Isto porque cada Registro fisico tem 8 bits (ou seja, um byte).
//=======================================================================

#define	total_DISPLAYS	8	// total de Displays de 7 segmentos.

#define	total_SAIDAS	16	// total de Saidas Digitais (multiplo de 8).

#define	total_ENTRADAS	16	// total de Entradas Digitais (multiplo de 8).

//***********************************************************************




//***********************************************************************
//	Define a polaridade do Display de 7 segmentos que e' usado no
//	Hardware deste Sistema, ou seja, Katodo ou Anodo Comum:
//=======================================================================

#define	tipo_DISPLAY	0	// 0 = Katodo Comum,  1 = Anodo Comum.

//***********************************************************************




//***********************************************************************
//	Definicao do estado fisico necessario para ligar um LED neste
//	Sistema:
//=======================================================================

#define	LED_ON	HIGH	// estado para ligar um LED no Hardware.

//***********************************************************************




//***********************************************************************
//	Define o C.I. usado como Registro de saida no Hardware, de
//	acordo com as opcoes a seguir:
//
//	- se for usado 74HC594, setar "CI_REGISTRO" como 594.
//	- se for usado 74HC595, setar "CI_REGISTRO" como 595.
//=======================================================================

#define	CI_REGISTRO	595	// define o C.I. de Registro usado no Hardware.

//***********************************************************************




//***********************************************************************
//	Definicao dos pinos do Arduino que sao usados para os sinais
//	"SCLK", "SDOUT", e "SDIN":
//=======================================================================

#define pino_SCLK	3	// pino do Arduino para o sinal "SCLK".

#define pino_SDOUT	4	// pino do Arduino para o sinal "SDOUT".

#define pino_SDIN	5	// pino do Arduino para o sinal "SDIN".

//***********************************************************************




//***********************************************************************
//	Definicao dos pinos do Arduino que sao usados para os sinais
//	"DISPLAY_LD", "OUTPUT_LD", e "INPUT_LD":
//=======================================================================

#define pino_INPUT_LD	6	// pino do Arduino para o sinal "INPUT_LD".

#define pino_OUTPUT_LD	7	// pino do Arduino para o sinal "OUTPUT_LD".

#define pino_DISPLAY_LD	8	// pino do Arduino para o sinal "DISPLAY_LD".

//***********************************************************************




//***********************************************************************
//	Definicao do pino do Arduino que habilita as saidas dos
//	Registros fisicos implementados no Hardware deste Sistema
//	via sinal "OUTS_ON":
//=======================================================================

#define pino_OUTS_ON	9	// pino do Arduino para o sinal "OUTS_ON".

//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Definicoes de padroes binarios para codificacoes genericas.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Definicao de padroes de bit para facilitar codificacao do
//	Programa em acessos individuais aos respectivos bits:
//=======================================================================

#define	BIT_0	0x01	// padrao binario correspondente ao "bit 0".
#define	BIT_1	0x02	// padrao binario correspondente ao "bit 1".
#define	BIT_2	0x04	// padrao binario correspondente ao "bit 2".
#define	BIT_3	0x08	// padrao binario correspondente ao "bit 3".
#define	BIT_4	0x10	// padrao binario correspondente ao "bit 4".
#define	BIT_5	0x20	// padrao binario correspondente ao "bit 5".
#define	BIT_6	0x40	// padrao binario correspondente ao "bit 6".
#define	BIT_7	0x80	// padrao binario correspondente ao "bit 7".

#define	BIT_blank	0x00	// padrao para todos os bits em "0".

//***********************************************************************




//***********************************************************************
//	Tabela com padroes de bits para acesso indexado pelo proprio
//	numero do bit:
//=======================================================================

const byte TAB_bits [] =

	{ BIT_0, BIT_1, BIT_2, BIT_3, BIT_4, BIT_5, BIT_6, BIT_7 };

//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Definicoes de padroes binarios para codificacao em 7 segmentos.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Mapeamento dos segmentos do Display conforme a ligacao fisica
//	dos pinos (bits) do HC594 (ou HC595) aos respectivos segmentos:
//=======================================================================

#define	SEG_a	BIT_0	// bit correspondente ao Segmento "a" do Display.
#define	SEG_b	BIT_1	// bit correspondente ao Segmento "b" do Display.
#define	SEG_c	BIT_2	// bit correspondente ao Segmento "c" do Display.
#define	SEG_d	BIT_3	// bit correspondente ao Segmento "d" do Display.
#define	SEG_e	BIT_4	// bit correspondente ao Segmento "e" do Display.
#define	SEG_f	BIT_5	// bit correspondente ao Segmento "f" do Display.
#define	SEG_g	BIT_6	// bit correspondente ao Segmento "g" do Display.

//***********************************************************************




//***********************************************************************
//	Codificacao dos digitos "BCD" para o padrao de 7 segmentos:
//=======================================================================

#define	Dig_0	( SEG_a + SEG_b + SEG_c + SEG_d + SEG_e + SEG_f )
#define	Dig_1	( SEG_b + SEG_c )
#define	Dig_2	( SEG_a + SEG_b + SEG_d + SEG_e + SEG_g )
#define	Dig_3	( SEG_a + SEG_b + SEG_c + SEG_d + SEG_g )
#define	Dig_4	( SEG_b + SEG_c + SEG_f + SEG_g )
#define	Dig_5	( SEG_a + SEG_c + SEG_d + SEG_f + SEG_g )
#define	Dig_6	( SEG_a + SEG_c + SEG_d + SEG_e + SEG_f + SEG_g )
#define	Dig_7	( SEG_a + SEG_b + SEG_c )
#define	Dig_8	( SEG_a + SEG_b + SEG_c + SEG_d + SEG_e + SEG_f + SEG_g )
#define	Dig_9	( SEG_a + SEG_b + SEG_c + SEG_d + SEG_f + SEG_g )

//-----------------------------------------------------------------------

#define Digit_blank	0x00	// padrao que apaga todos os segmentos.

//***********************************************************************




//***********************************************************************
//	Tabela com a codificacao dos digitos de 0 a 9 em padroes de
//	7 segmentos, para acesso indexado via proprio valor do digito:
//=======================================================================

const byte TAB_7seg [] =

	{ Dig_0, Dig_1, Dig_2, Dig_3, Dig_4, Dig_5, Dig_6, Dig_7, Dig_8, Dig_9 };

//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Definicao das variaveis correspondentes aos Registros fisicos
//	implementados no Hardware. No codigo estas variaveis sao
//	chamadas de Registros "virtuais", ja' que "espelham" o estado
//	fisico dos Registros do Hardware.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Definicao de uma variavel para Registro "virtual" do conteudo
//	dos Displays:
//=======================================================================

byte	vetor_DISPLAYS [total_DISPLAYS]; // Registro "virtual" dos Displays.

//***********************************************************************




//***********************************************************************
//	Definicao de uma variavel para Registro "virtual" do estado
//	das Saidas Digitais:
//=======================================================================

#define	n_reg_SAIDAS	total_SAIDAS/8	// total de Registros fisicos das Saidas.

byte	vetor_SAIDAS [n_reg_SAIDAS];	// Registro "virtual" das Saidas.

//***********************************************************************




//***********************************************************************
//	Definicao de uma variavel para Registro "virtual" do estado
//	das Entradas digitais:
//=======================================================================

#define	n_reg_ENTRADAS	total_ENTRADAS/8	// total de Registros fisicos das Entradas.

byte	vetor_ENTRADAS [n_reg_ENTRADAS];	// Registro "virtual" das Entradas.

//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes basicas de I/O para atualizacao dos Displays fisicos
//	implementados neste Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Selecao do nivel logico para ligar um segmento de um Display
//	fisico, conforme a especificacao de Katodo ou Anodo comum:
//=======================================================================

#if ( tipo_DISPLAY == 0 ) // se usando Katodo Comum:

#define SEGMENTO_ON	HIGH	// estado logico para ligar um segmento.

#endif

//-----------------------------------------------------------------------

#if ( tipo_DISPLAY == 1 ) // se usando Anodo Comum:

#define SEGMENTO_ON	LOW	// estado logico para ligar um segmento.

#endif

//***********************************************************************




//***********************************************************************
//	Funcao para serializacao de um padrao binario especificado
//	(em 7 segmentos), para o Registro fisico do Display. Devido
//	a serem 7 segmentos, um byte e' necessario para represnetar
//	estes segmentos.
//=======================================================================

void	DISPLAY_SERIAL_OUT ( byte DATA_7SEG )
{
//--------------------------------------------------
byte bit_n;	// bit atual sendo enviado ao ShiftRegister.
byte bit_MASK;	// mascara binaria para o bit atual.
//--------------------------------------------------

	bit_MASK = BIT_7;	// seleciona o bit inicial.

	for ( bit_n = 0; bit_n <= 7; bit_n++ ) // para cada bit do padrao binario:
	{
		if ( DATA_7SEG & bit_MASK )	// se o bit for "1":
		{
			digitalWrite ( pino_SDOUT, SEGMENTO_ON ); // entao liga o segmento.
		}
		else	digitalWrite ( pino_SDOUT, !SEGMENTO_ON ); // senao desliga o segmento.


		digitalWrite ( pino_SCLK, LOW ); // prepara para enviar o bit.

		delayMicroseconds(1);	// garante o "setup time".

		digitalWrite ( pino_SCLK, HIGH ); // envia o bit atual.

		delayMicroseconds(1);	// garante o "hold time".

		bit_MASK = ( bit_MASK >> 1 );	// seleciona proximo bit.
	}
}
//***********************************************************************




//***********************************************************************
//	Atualiza o Registro fisico do Display com o conteudo "virtual",
//	ou seja, com o conteudo da variavel de Registro esepcificada.
//	A sequencia de envio do conteudo da variavel de Registro, deve
//	estar conforme as ligacoes fisicas no Hardware.
//=======================================================================

void	atualizar_DISPLAY ( byte *REG_DISPLAY )
{
//--------------------------------------------------
byte n;
//--------------------------------------------------

	digitalWrite ( pino_DISPLAY_LD, LOW );	// "pre-setting" do PCLK.

	for ( n=total_DISPLAYS; n>0; n-- )	// para cada Display:
	{
		DISPLAY_SERIAL_OUT (  REG_DISPLAY [n-1] ); // envia o conteudo do Display "n".
	}

	digitalWrite ( pino_DISPLAY_LD, HIGH );	// atualiza o Registro fisico do Display.
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes basicas de I/O para atualizacao das Saidas fisicos
//	implementadas neste Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Funcao para serializacao de um padrao binario especificado,
//	para o Registro fisico das Saidas Digitais. A funcao serializa
//	8 bits, o que implica que um byte e' necessario para representar
//	o estado de 8 Saidas.
//=======================================================================

void	SAIDAS_SERIAL_OUT ( byte DATA_8 )
{
//--------------------------------------------------
byte bit_n;	// bit atual sendo enviado ao ShiftRegister.
byte bit_MASK;	// mascara binaria para o bit atual.
//--------------------------------------------------

	bit_MASK = BIT_7;	// seleciona o bit inicial.

	for ( bit_n = 0; bit_n <= 7; bit_n++ ) // para cada bit do padrao binario:
	{
		if ( DATA_8 & bit_MASK )	// se o bit for "1":
		{
			digitalWrite ( pino_SDOUT, HIGH ); // seta a Saida em "1".
		}
		else	digitalWrite ( pino_SDOUT, LOW ); // senao, seta a Saida em "0".


		digitalWrite ( pino_SCLK, LOW ); // prepara para enviar o bit.

		delayMicroseconds(1);	// garante o "setup time".

		digitalWrite ( pino_SCLK, HIGH ); // envia o bit atual.

		delayMicroseconds(1);	// garante o "hold time".

		bit_MASK = ( bit_MASK >> 1 );	// seleciona proximo bit.
	}
}
//***********************************************************************




//***********************************************************************
//	Atualiza o Registro fisico das Saidas Digitais com o conteudo
//	do Registro "virtual", ou seja, com o conteudo da variavel de
//	Registro especificada. A sequencia de envio do conteudo da
//	variavel de Registro, deve estar conforme as ligacoes fisicas
//	no Hardware.
//=======================================================================

void	atualizar_SAIDAS ( byte *REG_SAIDAS )
{
//--------------------------------------------------
byte n;
//--------------------------------------------------

	digitalWrite ( pino_OUTPUT_LD, LOW );	// "pre-setting" do PCLK.

	for ( n = n_reg_SAIDAS; n>0; n-- )	// para cada Registro de Saida:
	{
		SAIDAS_SERIAL_OUT ( REG_SAIDAS [n-1] ); // envia o conteudo do Registro "n".
	}

	digitalWrite ( pino_OUTPUT_LD, HIGH );	// atualiza o Registro fisico das Saidas.
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes basicas de I/O para atualizacao dos estados fisicos
//	das Entradas Digitais implementadas neste Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Funcao para deserializacao do Registro fisico das Entradas
//	Digitais, resultando em um padrao binario correspondente a
//	estas entradas. A funcao deserializa 8 bits, o que implica
//	que um byte e' necessario para retornar o padrao binario
//	resultante.
//=======================================================================

byte	ENTRADAS_SERIAL_IN ()
{
//--------------------------------------------------
byte bit_n;	// bit atual sendo lido do ShiftRegister.
byte DATA_IN;	// byte lido do ShiftRegister.
//--------------------------------------------------

	DATA_IN = 0;	// inicialmente, reseta o padrao binario.

	for ( bit_n = 0; bit_n <= 7; bit_n++ )	// para cada bit do padrao binario:
	{
		DATA_IN = ( DATA_IN << 1 );	// prepara para inserir o bit atual.

		DATA_IN += ( BIT_0 & digitalRead ( pino_SDIN ) ); // le e insere o bit atual.

		digitalWrite ( pino_SCLK, LOW ); // prepara para ler o proximo bit.

		delayMicroseconds(1);	// garante o "width time" do "SCLK".

		digitalWrite ( pino_SCLK, HIGH ); // cadencia o proximo bit.

		delayMicroseconds(1);	// garante o "width time" do "SCLK".
	}

	return ( DATA_IN );	// retorna o padrao binario lido.
}
//***********************************************************************




//***********************************************************************
//	Atualiza o Registro "virtual" especificado, com o estado das
//	Entradas Digitais. A sequencia de leitura das entradas deve
//	estar conforme as ligacoes fisicas no Hardware.
//=======================================================================

void	atualizar_ENTRADAS ( byte *REG_ENTRADAS )
{
//--------------------------------------------------
byte n;
//--------------------------------------------------

	digitalWrite ( pino_INPUT_LD, LOW ); // "pre-setting" para o "Load".
	digitalWrite ( pino_SCLK, LOW );

	delayMicroseconds(1);	// garante o "setup time".

	digitalWrite ( pino_SCLK, HIGH ); // faz o "Load" das Entradas.

	delayMicroseconds(1);	// garante o "hold time".

	digitalWrite ( pino_INPUT_LD, HIGH );	// "pre-setting" para o "shift".


	for ( n = 0; n < n_reg_ENTRADAS; n++ )	// para cada Registro de Entrada:
	{
		REG_ENTRADAS [n] = ENTRADAS_SERIAL_IN (); // le o Registro de Entrada "n".
	}
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes para alteracao dos Displays do Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Desliga todos os Displays. Isto e' feito apenas colocando em
//	"0" todos os bits do Registro "virtual" (variavel de Registro).
//	Apos a alteracao do Registro virtual, o Display fisico e' entao
//	atualizado.
//=======================================================================

void	desliga_DISPLAY ()
{
//--------------------------------------------------
byte n;
//--------------------------------------------------

	for ( n=0; n < total_DISPLAYS; n++ )	// para cada Display:
	{
		vetor_DISPLAYS [n] = BIT_blank;	// seta em "0" os bits do Display "n".
	}

	atualizar_DISPLAY ( vetor_DISPLAYS );	// atualiza o Display fisico.
}
//***********************************************************************




//***********************************************************************
//	Desliga apenas o Display "n" especificado. Isto e' feito apenas
//	colocando em "0" todos os bits correspondentes ao Display "n"
//	no Registro "virtual" (variavel de Registro).
//	Apos a alteracao do Registro virtual, o Display fisico e' entao
//	atualizado.
//=======================================================================

void	desliga_DISPLAY_n ( byte n )
{
	if ( ( n > 0 ) && ( n <= total_DISPLAYS ) ) // se o Display e' valido:
	{
		vetor_DISPLAYS [n-1] = BIT_blank; // seta em "0" os bits do Display "n".

		atualizar_DISPLAY ( vetor_DISPLAYS ); // atualiza o Display fisico.
	}
}
//***********************************************************************




//***********************************************************************
//	Altera apenas o Display "n" especificado, com o digito decimal
//	especificado (de 0 a 9). Isto e' feito apenas setando os bits
//	correspondentes ao Display "n" no Registro "virtual" (variavel
//	de Registro), com o padrao codificado em 7 segmentos do digito
//	decimal especificado. O padrao de bits e' obtido da Tabela de
//	padroes codificados em 7 segmentos.
//	Apos a alteracao do Registro virtual, o Display fisico e' entao
//	atualizado.
//=======================================================================

void	altera_DISPLAY_n ( byte n, byte digito )
{
	if ( ( n > 0 ) && ( n <= total_DISPLAYS ) ) // se o Display e' valido:
	{
		vetor_DISPLAYS [n-1] = TAB_7seg [digito]; // seta padrao de bits no Display "n".

		atualizar_DISPLAY ( vetor_DISPLAYS ); // atualiza o Display fisico.
	}
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes para alteracao das Saidas Digitais, e para leitura
//	das Entradas Digitais do Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Reseta todos as Saidas Digitais fisicas. Isto e' feito,
//	setando em "0" todos os bits correspondentes as Saidas no
//	Registro "virtual" (variavel de Registro).
//	Apos a alteracao do Registro virtual, o Registro fisico e'
//	atualizado.
//=======================================================================

void	desliga_SAIDAS ()
{
//--------------------------------------------------
byte n;
//--------------------------------------------------

	for ( n=0; n < n_reg_SAIDAS; n++ ) // para cada Registro de Saida:
	{
		vetor_SAIDAS [n] = 0;	// reseta o Registro "n".
	}

	atualizar_SAIDAS ( vetor_SAIDAS ); // atualiza o estado fisico das Saidas.
}
//***********************************************************************




//***********************************************************************
//	Reseta apenas a Saida Digital fisica "n" especificada.
//	Simplesmente o bit que corresponde a Saida especificada "n",
//	e' colocado em "0" no Registro "virtual" (variavel de Registro).
//	Apos a alteracao do Registro virtual, o Registro fisico e'
//	atualizado.
//=======================================================================

void	reseta_SAIDA_n ( byte n )
{
//--------------------------------------------------
byte n_REG, n_bit;
//--------------------------------------------------

	if ( ( n >= 1 ) && ( n <= total_SAIDAS ) )	// se "n" e' valido:
	{
		n_REG = (n-1) / 8;	// obtem o numero do Registro da Saida.

		n_bit = (n-1) & 0x07;	// obtem o numero do bit no Registro da Saida.

		vetor_SAIDAS [n_REG] &= ~TAB_bits [n_bit];	// reseta o bit no "Registro Virtual".

		atualizar_SAIDAS ( vetor_SAIDAS ); // atualiza o estado fisico das Saidas.
	}
}
//***********************************************************************




//***********************************************************************
//	Seta apenas a Saida Digital fisica "n" especificada.
//	Simplesmente o bit que corresponde a Saida especificada "n",
//	e' colocado em "1" no Registro "virtual" (variavel de Registro).
//	Apos a alteracao do Registro virtual, o Registro fisico e'
//	atualizado.
//=======================================================================

void	seta_SAIDA_n ( byte n )
{
//--------------------------------------------------
byte n_REG, n_bit;
//--------------------------------------------------

	if ( ( n >= 1 ) && ( n <= total_SAIDAS ) )	// se "n" e' valido:
	{
		n_REG = (n-1) / 8;	// obtem o numero do Registro da Saida.

		n_bit = (n-1) & 0x07;	// obtem o numero do bit no Registro da Saida.

		vetor_SAIDAS [n_REG] |= TAB_bits [n_bit];	// seta o bit no "Registro Virtual".

		atualizar_SAIDAS ( vetor_SAIDAS ); // atualiza o estado fisico das Saidas.
	}
}
//***********************************************************************




//***********************************************************************
//	Le o estado da Entrada Digital "n" especificada. Para isso,
//	o Registro virtual (variavel de Registro) e' atualizado com
//	o estado fisico atual das entradas. O estado retornado e'
//	do tipo booleano, implicando que "HIGH" (ou "true") corresponde
//	a uma entrada em nivel logico "1", e que "LOW" (ou "false")
//	corresponde a uma entrada em nivel logico "0".
//=======================================================================

bool	le_ENTRADA_n ( byte n )
{
//--------------------------------------------------
bool estado;
byte n_REG, n_bit;
//--------------------------------------------------

	if ( ( n >= 1 ) && ( n <= total_ENTRADAS ) )	// se "n" e' valido:
	{
		atualizar_ENTRADAS ( vetor_ENTRADAS );	// atualiza o "Registro Virtual" das Entradas.

		n_REG = (n-1) / 8;	// obtem o numero do Registro da Entrada.

		n_bit = (n-1) & 0x07;	// obtem o numero do bit no Registro da Entrada.

		estado = vetor_ENTRADAS [n_REG] & TAB_bits [n_bit]; // obtem estado da Entrada "n".
	}
	else	estado = false;	// resultado, se "n" e' invalido.


	return ( estado );	// retorna o estado da Entrada "n".
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes para inicializacao do Hardware deste Sistema.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Seleciona o nivel logico para habilitar os Registros fisicos,
//	conforme o C.I. de Registro que esta' sendo usado no Hardware
//	deste Sistema (74HC594 ou 74HC595):
//=======================================================================

#if ( CI_REGISTRO == 594 )	// se esta usando o 74HC594:

#define OUTS_ON	HIGH	// nivel logico que libera o sinal PCLR# do HC594.

#endif

//-----------------------------------------------------------------------

#if ( CI_REGISTRO == 595 )	// se esta usando o 74HC595:

#define OUTS_ON	LOW	// nivel logico que habilita o sinal OE# do HC595.

#endif

//***********************************************************************




//***********************************************************************
//	Funcao para "desabilitar" as saidas dos Registros fisicos que
//	estao implementados no Hardware deste Sistema. Para o 74HC595,
//	as saidas sao desabilitadas (colocadas em "tri-state"), e o
//	conteudo dos Registros permanece intacto. Para o 74HC594, as
//	saidas sao setadas em "0", pois o conteudo dos Registros e'
//	zerado.
//=======================================================================

void	desabilita_REGISTROS_PIO ()
{
	digitalWrite ( pino_OUTS_ON, !OUTS_ON ); // desabilita Registros de HW.
}
//***********************************************************************




//***********************************************************************
//	Funcao para "habilitar" as saidas dos Registros fisicos que
//	estao implementados no Hardware deste Sistema. Esta funcao so'
//	deve ser chamada depois que os sinais de controle do Hardware
//	e os Registros "virtuais" tiverem sido inicializados. Se o C.I.
//	de Registro e' o 74HC595, entao deve-se tambem atualizar os
//	Registros fisicos com o conteudo dos respectivos Registros
//	"virtuais", antes de chamar esta funcao.
//=======================================================================

void	habilita_REGISTROS_PIO ()
{
	digitalWrite ( pino_OUTS_ON, OUTS_ON ); // habilita Registros de HW.
}
//***********************************************************************




//***********************************************************************
//	Inicializa os sinais de controle do Hardware deste Sistema.
//	Para isto, sao inicializados e configurados os pinos que
//	correspondem aos respectivos sinais no Hardware.
//
//	Obs.: NAO deve ser alterada a sequencia de inicializacao que
//	e' executada na funcao.
//=======================================================================

void	inicializa_HARDWARE_PIO ()
{
	desabilita_REGISTROS_PIO ();		// garante Registros desabilitados.
	pinMode ( pino_OUTS_ON, OUTPUT );	// configura o pino do sinal "OUTS_ON".

//..................................................

	digitalWrite ( pino_SDOUT, LOW );	// estado inicial do sinal "SDOUT".
	digitalWrite ( pino_SCLK, LOW );	// estado inicial do sinal "SCLK".

	digitalWrite ( pino_DISPLAY_LD, LOW );	// estado inicial do sinal "DISPLAY_LD".
	digitalWrite ( pino_OUTPUT_LD, LOW );	// estado inicial do sinal "OUTPUT_LD".
	digitalWrite ( pino_INPUT_LD, LOW );	// estado inicial do sinal "INPUT_LD".

//..................................................

	pinMode ( pino_SDOUT, OUTPUT );		// configura o pino do sinal "SDOUT".
	pinMode ( pino_SCLK, OUTPUT );		// configura o pino do sinal "SCLK".

	pinMode ( pino_DISPLAY_LD, OUTPUT );	// configura o pino do sinal "DISPLAY_LD".
	pinMode ( pino_OUTPUT_LD, OUTPUT );	// configura o pino do sinal "OUTPUT_LD".
	pinMode ( pino_INPUT_LD, OUTPUT );	// configura o pino do sinal "INPUT_LD".

//..................................................

	pinMode ( pino_SDIN, INPUT );	// configura o pino do sinal "SDIN".
}
//***********************************************************************




//***********************************************************************
//	Inicializa o Hardware deste Sistema. Primeiro sao inicializados
//	e configurados os sinais de controle do Hardware. Em seguida,
//	sao zerados os Registros "virtuais" dos Displays e das Saidas
//	Digitais, sendo tambem atualizado o Registro "virtual" das
//	Entradas Digitais. Finalmente, sao habilitadas as saidas dos
//	Registros fisicos.
//
//	Obs.: NAO deve ser alterada a sequencia de inicializacao que
//	e' executada na funcao.
//=======================================================================

void	inicializa_SISTEMA_PIO ()
{
	inicializa_HARDWARE_PIO ();	// inicializa os sinais de controle do Hardware.

//..................................................

	desliga_DISPLAY ();	// zera o Registro "virtual" do Display.

	desliga_SAIDAS ();	// zera o Registro "virtual" das Saidas Digitais.

	atualizar_ENTRADAS ( vetor_ENTRADAS );	// atualiza o Registro das Entradas Digitais.

//..................................................

	habilita_REGISTROS_PIO ();	// habilita as saidas dos Registros fisicos.
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes para exibicao de valores decimais inteiros nos
//	Displays, usando as funcoes da BIOS do Sistema Multi-PIO.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Funcao para converter um valor binario de 16 bits, para um
//	valor "decimal codificado em binario" (BCD). Como cada digito
//	"BCD" ocupa 4 bits, entao o maximo valor resultante de uma
//	conversao sera' 9999, que e' portanto o limite numerico para
//	uma conversao correta. Valores acima deste limite, terao o
//	zero como resultado. A tecnica classica de conversao e' usada,
//	ou seja, divisoes sucessivas por 10, para se obter cada um dos
//	digitos BCD.
//=======================================================================

word	HEX16_to_BCD16 ( word HEX_16 )
{
//--------------------------------------------------
word	BCD_result;
byte	i;
//--------------------------------------------------

	BCD_result = 0;	// valor inicial da conversao (somatorio).

	if ( HEX_16 < 10000 )	// se esta' dentro do limite conversivel:
	{
		i = 0;	// contador de deslocamentos, para "pesagem" dos digitos.

		while ( HEX_16 >= 10 )	// enquanto for possivel dividir por 10:
		{
			BCD_result += ( ( HEX_16 % 10 ) << i );	// calcula e acrescenta um digito BCD.

			HEX_16 /= 10;	// valor inteiro para obter proximo digito BCD.

			i += 4;	// obtem o "peso" para o proximo digito BCD.
		}

		BCD_result += ( HEX_16 << i );	// acrescenta o digito BCD menos significativo.
	}

	return ( BCD_result );	// retorna o resultado convertido para BCD.
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 2 digitos (ou seja, que
//	esteja entre 0 e 99), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 2 digitos BCD, a partir do Display "n" especificado.
//=======================================================================

void	mostra_valor_2_Digitos ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	altera_DISPLAY_n ( pos_DISPLAY, ( BCD & 0x0F ) ); // exibe o digito 1, no Display "n".

	altera_DISPLAY_n ( (pos_DISPLAY+1), ( ( BCD >> 4 ) & 0x0F ) );	// exibe o digito 2, no Display "n+1".
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 2 digitos (ou seja, que
//	esteja entre 0 e 99), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 2 digitos BCD, a partir do Display "n" especificado.
//	Antes de exibir o segundo digito BCD, a rotina verifica se ele
//	e' zero, e caso seja, o digito nao e' exibido, sendo apagado o
//	Display correspondente. Como resultado, um zero nao significativo
//	("trailing zero"), nao e' exibido.
//=======================================================================

void	mostra_valor_2_Digitos_sem_zeros ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
byte digito1, digito2;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	digito1 = ( BCD & 0x0F );	// obtem o digito "1".
	digito2 = ( BCD >> 4 ) & 0x0F;	// obtem o digito "2".
//..................................................

	altera_DISPLAY_n ( pos_DISPLAY, digito1 ); // exibe o digito "1".
//..................................................

	if ( digito2 != 0 )	// se o digito "2" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+1), digito2 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+1);	// senao apaga este digito.
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 3 digitos (ou seja, que
//	esteja entre 0 e 999), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 3 digitos BCD, a partir do Display "n" especificado.
//=======================================================================

void	mostra_valor_3_Digitos ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	altera_DISPLAY_n ( pos_DISPLAY, ( BCD & 0x0F ) ); // exibe o digito 1, no Display "n".

	altera_DISPLAY_n ( (pos_DISPLAY+1), ( ( BCD >> 4 ) & 0x0F ) );	// exibe o digito 2, no Display "n+1".

	altera_DISPLAY_n ( (pos_DISPLAY+2), ( ( BCD >> 8 ) & 0x0F ) );	// exibe o digito 3, no Display "n+2".
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 3 digitos (ou seja, que
//	esteja entre 0 e 999), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 3 digitos BCD, a partir do Display "n" especificado.
//	Antes de exibir um digito BCD no Display, a rotina verifica se
//	ele e' zero e se os digitos mais signficativos tambem sao zero,
//	e caso sejam, o digito nao e' exibido, sendo apagado o Display
//	correspondente. Como resultado, os zeros nao significativos
//	("trailing zeros"), nao sao exibidos.
//=======================================================================

void	mostra_valor_3_Digitos_sem_zeros ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
byte digito1, digito2;
byte digito3;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	digito1 = ( BCD & 0x0F );	// obtem o digito "1".
	digito2 = ( BCD >> 4 ) & 0x0F;	// obtem o digito "2".
	digito3 = ( BCD >> 8 ) & 0x0F;	// obtem o digito "3".
//..................................................

	altera_DISPLAY_n ( pos_DISPLAY, digito1 ); // exibe o digito "1".
//..................................................

	if ( ( digito3 != 0 ) || ( digito2 != 0 ) )	// se o digito "2" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+1), digito2 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+1);	// senao apaga este digito.
//..................................................

	if ( digito3 != 0 )	// se o digito "3" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+2), digito3 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+2);	// senao apaga este digito.
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 4 digitos (ou seja, que
//	esteja entre 0 e 9999), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 4 digitos BCD, a partir do Display "n" especificado.
//=======================================================================

void	mostra_valor_4_Digitos ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	altera_DISPLAY_n ( pos_DISPLAY, ( BCD & 0x0F ) ); // exibe o digito 1, no Display "n".

	altera_DISPLAY_n ( (pos_DISPLAY+1), ( ( BCD >> 4 ) & 0x0F ) );	// exibe o digito 2, no Display "n+1".

	altera_DISPLAY_n ( (pos_DISPLAY+2), ( ( BCD >> 8 ) & 0x0F ) );	// exibe o digito 3, no Display "n+2".

	altera_DISPLAY_n ( (pos_DISPLAY+3), ( ( BCD >> 12 ) & 0x0F ) );	// exibe o digito 4, no Display "n+3".
}
//***********************************************************************




//***********************************************************************
//	Exibe no Display um valor decimal de 4 digitos (ou seja, que
//	esteja entre 0 e 9999), a partir do Display "n" especificado.
//	Para isto a rotina converte pra BCD, o valor binario fornecido
//	como parametro de entrada, e em seguida exibe no Display cada
//	um dos 4 digitos BCD, a partir do Display "n" especificado.
//	Antes de exibir um digito BCD no Display, a rotina verifica se
//	ele e' zero e se os digitos mais signficativos tambem sao zero,
//	e caso sejam, o digito nao e' exibido, sendo apagado o Display
//	correspondente. Como resultado, os zeros nao significativos
//	("trailing zeros"), nao sao exibidos.
//=======================================================================

void	mostra_valor_4_Digitos_sem_zeros ( word valor , byte pos_DISPLAY )
{
//--------------------------------------------------
word BCD;
byte digito1, digito2;
byte digito3, digito4;
//--------------------------------------------------

	BCD = HEX16_to_BCD16 ( valor );	// obtem o valor BCD correspondente.

	digito1 = ( BCD & 0x0F );	// obtem o digito "1".
	digito2 = ( BCD >> 4 ) & 0x0F;	// obtem o digito "2".
	digito3 = ( BCD >> 8 ) & 0x0F;	// obtem o digito "3".
	digito4 = ( BCD >> 12 ) & 0x0F;	// obtem o digito "4".
//..................................................

	altera_DISPLAY_n ( pos_DISPLAY, digito1 ); // exibe o digito "1".
//..................................................

	if ( ( digito4 != 0 ) || ( digito3 != 0 ) || ( digito2 != 0 ) )	// se o digito "2" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+1), digito2 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+1);	// senao apaga este digito.
//..................................................

	if ( ( digito4 != 0 ) || ( digito3 != 0 ) )	// se o digito "3" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+2), digito3 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+2);	// senao apaga este digito.
//..................................................

	if ( digito4 != 0 )	// se o digito "4" e' siginificativo:
	{
		altera_DISPLAY_n ( (pos_DISPLAY+3), digito4 );	// entao exibe este digito.
	}
	else	desliga_DISPLAY_n (pos_DISPLAY+3);	// senao apaga este digito.
}
//***********************************************************************




//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//
//	Funcoes "setup" e "loop" da Plataforma do Arduino.
//
//\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\




//***********************************************************************
//	Inicializa este Sistema.
//=======================================================================

void	setup ()
{
	inicializa_SISTEMA_PIO (); // inicializa o Hardware do Multi-PIO.

//..................................................

//	insira aqui o codigo do "setup", para o seu Sistema.

}
//***********************************************************************




//***********************************************************************
//	Loop principal do Sistema: sao executadas varias sequencias
//	de controle de acionamento dos LEDs e Displays, para efeito de
//	demonstracao do uso das diversas funcoes implementadas na BIOS
//	do Sistema Multi-PIO.
//=======================================================================

void	loop ()
{
//--------------------------------------------------
byte N, digito;
word CONT;
//--------------------------------------------------

	desliga_DISPLAY (); // desliga todos os Displays.
	desliga_SAIDAS (); // desliga todos as Saidas (e LEDs ligados a estas).

//..................................................
// demonstracao: vai ligando sequencialmente os LEDs, do "1" ao "8":

	for ( N=1; N <= 8; N++ ) // para cada um dos LEDs nas saidas "1" a "8":
	{
		seta_SAIDA_n ( N );	// liga a saida, e assim o LED conectado nela.

		delay(500);	// tempo que fica ligado.
	}

//..................................................
// demonstracao: vai desligando sequencialmente os LEDs, do "1" ao "8":

	for ( N=1; N <= 8; N++ ) // para cada um dos LEDs nas saidas "1" a "8":
	{
		reseta_SAIDA_n ( N );	// desliga a saida, e assim o LED conectado nela.

		delay(500);	// tempo que fica desligado.
	}

//..................................................
// demonstracao: vai ligando e desligando sequencialmente os LEDs, do "1" ao "8":

	for ( N=1; N <= 8; N++ ) // para cada um dos LEDs nas saidas "1" a "8":
	{
		seta_SAIDA_n ( N );	// liga a saida, e assim o LED conectado nela.

		delay(500);	// tempo que fica ligado.

		reseta_SAIDA_n ( N );	// desliga a saida, e assim o LED conectado nela.
	}

//..................................................
// demonstracao: vai ligando e desligando sequencialmente os LEDs, do "8" ao "1":

	for ( N=8; N >= 1; N-- ) // para cada um dos LEDs nas saidas "8" a "1":
	{
		seta_SAIDA_n ( N );	// liga a saida, e assim o LED conectado nela.

		delay(500);	// tempo que fica ligado.

		reseta_SAIDA_n ( N );	// desliga a saida, e assim o LED conectado nela.
	}

//..................................................
// demonstracao: vai mostrando numeros crescentes nos Displays:

	for ( N=1; N <= total_DISPLAYS; N++ ) // para cada um dos Displays, do primeiro ao ultimo:
	{
		altera_DISPLAY_n ( N, N ); // exibe valor "n", no Display "n".

		delay(500);	// tempo de exibicao.
	}

//..................................................
// vdemonstracao: ai desligando os Displays, do ultimo ao primeiro:

	for ( N=total_DISPLAYS; N >= 1; N-- ) // para cada um dos Displays, do ultimo ao primeiro:
	{
		desliga_DISPLAY_n ( N ); // desliga o Display "n".

		delay(500);	// tempo de exibicao.
	}

//..................................................
// demonstracao: vai deslocando numeros nos Displays:

	for ( CONT=8; CONT >=1; CONT-- ) // para os digitos BCD "8" ao "1":
	{
		digito = CONT-1; // obtem o digito BCD logo abaixo do atual.

		for ( N=1; N<=total_DISPLAYS; N++) // para cada um dos Displays, do 1 ao 8:
		{
			if ( ( N+digito ) <= 8 ) // se o digito a ser exibido e' valido:
			{
				altera_DISPLAY_n ( N, N+digito ); // entao exibe conforme o Display.
			}
			else	desliga_DISPLAY_n ( N ); // senao apaga o Display.
		}

		delay(500);	// tempo de exibicao.
	}

	desliga_DISPLAY ();	// desliga todos os Displays.

	delay(500);	// tempo que ficam desligados.

//..................................................
// demonstracao: faz um contador de 0 a 99, nos Displays "1" e "2",
// usando funcao que sempre mostra os dois digitos:

	for ( CONT=0; CONT < 100; CONT++ ) // para uma contagem de 0 a 99:
	{
		mostra_valor_2_Digitos ( CONT , 1 ); // exibe esta contagem nos Displays 1 e 2.

		delay(125);	// tempo de exibicao.
	}

	delay(1000);	// tempo de exibicao da contagem final.

	desliga_DISPLAY ();	// desliga todos os Displays.

	delay(500);	// tempo que ficam desligados.

//..................................................
// demonstracao: faz um contador de 0 a 1234, nos Displays "3" ao "6",
// usando funcao que exclui os zeros sem significado:

	for ( CONT=0; CONT <= 1234; CONT++ ) // para uma contagem de 0 a 1234:
	{
		mostra_valor_4_Digitos_sem_zeros ( CONT , 3 ); // exibe esta contagem nos Displays 3 a 6.

		delay(10);	// tempo de exibicao.
	}

	delay(1000);	// tempo de exibicao da contagem final.

	desliga_DISPLAY ();	// desliga todos os Displays.

	delay(500);	// tempo que ficam desligados.

//..................................................
// demonstracao: Faz um "pisca-pisca" com os Displays e LEDs:

	for ( CONT=1; CONT <= 5; CONT++ ) // por 5 vezes:
	{
		for ( N=1; N <= 8; N++ ) // liga os LEDs nas saidas 1 a 8:
		{
			seta_SAIDA_n ( N );
		}

		for ( N=1; N <= total_DISPLAYS; N++ ) // exibe "8" em todos os Displays:
		{
			altera_DISPLAY_n ( N, 8 );
		}

		delay(500);	// tempo que todos LEDs e Displays ficam ligados.

		desliga_SAIDAS ();	// desliga todos os LEDs.
		desliga_DISPLAY ();	// desliga todos os Displays.

		delay(500);	// tempo que ficam desligados.
	}

//..................................................
// demonstracao: durante 15 segundos, copia para as 8 primeiras saidas,
// o estado das respectivas 8 primeiras entradas. Havendo LEDs nas
// saidas e chaves nas entradas, entao os LEDs ligam ou desligam conforme
// o estado "ON/OFF" das respectivas chaves:

	for ( CONT=1; CONT <= 150; CONT++ )	// por 150 loops:
	{
		for ( N=1; N <= 8; N++ )	// para as 8 primeiras entradas:
		{
			if ( le_ENTRADA_n ( N ) ) // se a entrada está em "1":
			{
				seta_SAIDA_n ( N ); // entao coloca em "1" a saida correspondente.
			}
			else	reseta_SAIDA_n ( N ); // senao coloca em "0" a saida correspondente.
		}

		delay(100);	// tempo que determina a velocidade de leitura das entradas.
	}

//..................................................
}
//***********************************************************************



